/*
    AQTRONIX C++ Library
    Copyright 2005-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// DNS.cpp: implementation of the CDNS class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DNS.h"

//old IPv4 implementation
#include <winsock.h>

/*
//IPv6 implementation
#include <winsock2.h>
#include <ws2tcpip.h>

// link with ws2_32.lib for getnameinfo
#pragma comment(lib, "Ws2_32.lib")
*/

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDNS::CDNS(LPCTSTR Server)
{
	DNSServer  = Server;
	//DNSBL: http://www.nsoftware.com/kb/tutorials/spamkiller.aspx
}

CDNS::~CDNS()
{

}

void CDNS::SetServer(LPCTSTR Server)
{
	DNSServer = Server;
}

bool CDNS::Lookup(LPCTSTR HostName, CStringArray& IPList)
{
	try{

		WSADATA sockData;
		hostent *dnsInfo;

		if(WSAStartup(0x0101, &sockData) != 0){
			//startup failed
			WSACleanup();
			return false;
		}else{

			//do lookup
			int errcode(0);
			dnsInfo = gethostbyname(HostName);
			errcode = WSAGetLastError();	//immediately after gethostbyname

			if(dnsInfo==NULL){
				//no data returned

				if(errcode==WSAHOST_NOT_FOUND || errcode==WSANO_DATA){	//no data available
					WSACleanup();
					return true;
				}else{								//other error
					WSACleanup();
					return false;
				}
			}else{
				//loop through all returned IP addresses
				for(int i=0; dnsInfo->h_addr_list[i]; i++)
				{
					//add them to the array + increase counter
					IPList.Add(inet_ntoa(*((struct in_addr *)dnsInfo->h_addr_list[i])));
				}
				//cleanup
				WSACleanup();
				return true;
			}
		}
	}catch(...){
		return false;
	}
}

bool CDNS::Lookup(IIPAddress& IP, CStringArray& HostNames)
{
	try{

		WSADATA sockData;
		hostent *dnsInfo = NULL;

		if(WSAStartup(0x0101, &sockData) != 0){
			//starting failed
  			WSACleanup(); 
			return false;
		}else{

			//lookup
			if(IP.GetSize()==IPV4_SIZE){

				unsigned long ip = INADDR_NONE;
				ip = inet_addr(IP.ToString());
				if(ip==INADDR_NONE || ip==INADDR_ANY){
					WSACleanup();
					return false;
				}

				dnsInfo = gethostbyaddr((char*)&ip,sizeof ip,AF_INET);

			}else if(IP.GetSize()==IPV6_SIZE){
				//TODO: DNS - IPv6 reverse lookup

				/*
				struct sockaddr_in saGNI;
				char hostname[NI_MAXHOST];
				char servInfo[NI_MAXSERV];
				u_short port = 27015;

				saGNI.sin_family = AF_INET;
				saGNI.sin_addr.s_addr = inet_addr(IP.ToString());
				saGNI.sin_port = htons(port);

				DWORD dwRetval = getnameinfo((struct sockaddr *) &saGNI,
                           sizeof (struct sockaddr),
                           hostname,
                           NI_MAXHOST, servInfo, NI_MAXSERV, NI_NUMERICSERV);

				if(dwRetval != 0)
					HostNames.Add(hostname);
				*/



				WSACleanup();
				return false;
			}else{
				WSACleanup();
				return false;
			}

			//dnsInfo = gethostbyname(IP.ToString());

			if(dnsInfo==NULL){
				//no data returned
				if(WSAGetLastError()==WSAHOST_NOT_FOUND){	//no data available
					WSACleanup();
					return true;
				}else{										//other error
					WSACleanup();
					return false;
				}
			}else{

				if(dnsInfo->h_name)
					HostNames.Add(dnsInfo->h_name);

				//cleanup
				WSACleanup();
				return true;
			}
		}
	}catch(...){
		return false;
	}
}

bool CDNS::MXLookup(LPCTSTR DomainName, CStringArray& IPList)
{
	try{

		//do MX lookup
		bool ret(false);
		PDNS_RECORD pDNS = NULL;
		PDNS_RECORD *ppQueryResultsSet = &pDNS;
		DNS_STATUS dnsResult;

		dnsResult = DnsQuery(DomainName,DNS_TYPE_MX,DNS_QUERY_STANDARD,NULL,ppQueryResultsSet,NULL);

		if(dnsResult == ERROR_SUCCESS){
			//found records
			DNS_RECORD* p = *ppQueryResultsSet;
			CString mx("");
			while(p != NULL){
				//only MX records
				if(p->wType == DNS_TYPE_MX){
					//p->dwTtl; //TTL
					//p->Data.MX.wPreference;	//Preference
					mx = p->Data.MX.pNameExchange;	//mx name
					if(mx.GetLength()>0 && Lookup(mx,IPList)){	//lookup
						ret = true;
					}
				}
				p = p->pNext;
			}

			//cleanup
			DnsRecordListFree(pDNS, DnsFreeRecordListDeep);
		}
		return ret;
	}catch(...){
		return false;
	}
}

bool CDNS::IsFQDN(CString &Name, bool EndingDot)
{
	bool ret(false);
	int l = Name.GetLength();
	//max 255 characters but not empty
	if(l<256 && l>0){
		//dotted
		if(Name.Find('.')!=-1){
			//is FQDN characters
			if(IsFQDNCharacters(Name)){
				//is not IP address
				if(!CIPAddress::IsIPAddress(Name)){
					ret = true;
				}
			}
		}
	}
	if(ret && EndingDot){
		if(Name.Right(1)!="."){
			return false;
		}
	}
	return ret;
}

bool CDNS::IsFQDNCharacters(CString &Name)
{
	//Strict RFC (ANSI) . Allows "A" to "Z", "a" to "z", 
	//the hyphen (-), the asterisk (*) as a first label; 
	//and the underscore (_) as the first character in a label.
	
	//Non RFC (ANSI) . Allows all characters from Strict RFC (ANSI),
	//and allows the underscore (_) anywhere in a name.

	//Multibyte (UTF-8) . Allows all characters from Non RFC (ANSI),
	//and allows UTF - 8 characters.

	//a to z, A to Z, 0 to 9, ".", and the "-" character
	//rfc extension for services SRV: "_" in beginning of label
	//and allow "_" in ansi names (non-rfc)
	//International domain names are encoded to FQDN characters
	int l = Name.GetLength();
	TCHAR c;
	for(int i=0;i<l;i++){
		c = Name[i];
		if(c!='.' && c!='-' && c!='_' && (c<45 || (c>57 && c<65) || (c>90 && c<97) || c> 122))
			return false;
	}

	return true;
}
