/*
	AQTRONIX XML Configuration Editor
	Copyright 2002-2015 Parcifal Aertssen

	This file is part of AQTRONIX XML Configuration Editor.

    AQTRONIX XML Configuration Editor is free software; you can redistribute
	it and/or modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

    AQTRONIX XML Configuration Editor is distributed in the hope that it will
	be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX XML Configuration Editor; if not, write to the 
	Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "stdafx.h"
#include "EditableListCtrl.h"
#include "InPlaceEdit.h"

#define IDC_IPEDIT 101 //see resources of Config project

CEditableListCtrl::CEditableListCtrl(void)
{
}

CEditableListCtrl::~CEditableListCtrl(void)
{
}

BEGIN_MESSAGE_MAP(CEditableListCtrl, CSortableListCtrl)
	//{{AFX_MSG_MAP(CEditableListCtrl)
	ON_NOTIFY_REFLECT(LVN_ENDLABELEDIT, OnEndlabeledit)
	ON_WM_KEYDOWN()
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
//CRUD operations

void CEditableListCtrl::NewItem(LPCTSTR msg) 
{
	int item = GetSelectionMark();
	if(item==-1)
		item = 0;
	InsertItem(item,msg);
	EditLabel(item);
}

void CEditableListCtrl::EditCurrent() 
{
	int item = GetSelectionMark();
	if(item!=-1)
		EditLabel(item);
}

void CEditableListCtrl::DeleteSelectedItems() 
{
	int count = GetSelectedCount();
	int item = -1;
	for(int i=0;i<count;i++){
		item = GetNextItem(-1,LVNI_SELECTED);
		DeleteItem(item);
	}
}

void CEditableListCtrl::SelectAll()
{
	SetItemState(-1,LVNI_SELECTED,LVNI_SELECTED);
}

/////////////////////////////////////////////////////////////////////////////
// CEditableListCtrl message handlers

void CEditableListCtrl::OnEndlabeledit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
	// Add your control notification handler code here
	
	LV_ITEM	*plvItem = &pDispInfo->item;

	if (plvItem->pszText != NULL)
	{
		SetItemText(plvItem->iItem, plvItem->iSubItem, plvItem->pszText);
	}
	*pResult = 0;

	//*pResult = 1;	//allow edits

	//*pResult = 0;
}

void CEditableListCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(nChar==VK_DELETE)
		DeleteSelectedItems();

	if(nChar==VK_F2)
		EditCurrent();
		
	CSortableListCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
}

int CEditableListCtrl::HitTestEx(CPoint &point, int *col) const
{
	/* Extended hittest for subitem edit
	 * Function from: www.codeguru.com
	 */

	int colnum = 0;
	int row = HitTest( point, NULL );
	
	if(col) *col = 0;

	// Make sure that the ListView is in LVS_REPORT
	if( (GetWindowLong(m_hWnd, GWL_STYLE) & LVS_TYPEMASK) != LVS_REPORT )
		return row;

	// Get the top and bottom row visible
	row = GetTopIndex();
	int bottom = row + GetCountPerPage();
	if( bottom > GetItemCount() )
		bottom = GetItemCount();
	
	// Get the number of columns
	CHeaderCtrl* pHeader = (CHeaderCtrl*)GetDlgItem(0);
	int nColumnCount = pHeader->GetItemCount();

	// Loop through the visible rows
	for( ;row <=bottom;row++)
	{
		// Get bounding rect of item and check whether point falls in it.
		CRect rect;
		GetItemRect( row, &rect, LVIR_BOUNDS );
		if( rect.PtInRect(point) )
		{
			// Now find the column
			for( colnum = 0; colnum < nColumnCount; colnum++ )
			{
				int colwidth = GetColumnWidth(colnum);
				if( point.x >= rect.left 
					&& point.x <= (rect.left + colwidth ) )
				{
					if( col ) *col = colnum;
					return row;
				}
				rect.left += colwidth;
			}
		}
	}
	return -1;
}

CEdit* CEditableListCtrl::EditSubLabel(int nItem, int nCol)
{
	/* Edit subitem
	 * Function from: www.codeguru.com
	 */

	// The returned pointer should not be saved

	// Make sure that the item is visible
	if( !EnsureVisible( nItem, TRUE ) ) return NULL;

	// Make sure that nCol is valid
	CHeaderCtrl* pHeader = (CHeaderCtrl*)GetDlgItem(0);
	int nColumnCount = pHeader->GetItemCount();
	if( nCol >= nColumnCount || GetColumnWidth(nCol) < 5 )
		return NULL;

	// Get the column offset
	int offset = 0;
	for( int i = 0; i < nCol; i++ )
		offset += GetColumnWidth( i );

	CRect rect;
	GetItemRect( nItem, &rect, LVIR_BOUNDS );

	// Now scroll if we need to expose the column
	CRect rcClient;
	GetClientRect( &rcClient );
	if( offset + rect.left < 0 || offset + rect.left > rcClient.right )
	{
		CSize size;
		size.cx = offset + rect.left;
		size.cy = 0;
		Scroll( size );
		rect.left -= size.cx;
	}

	// Get Column alignment
	LV_COLUMN lvcol;
	lvcol.mask = LVCF_FMT;
	GetColumn( nCol, &lvcol );
	DWORD dwStyle ;
	if((lvcol.fmt&LVCFMT_JUSTIFYMASK) == LVCFMT_LEFT)
		dwStyle = ES_LEFT;
	else if((lvcol.fmt&LVCFMT_JUSTIFYMASK) == LVCFMT_RIGHT)
		dwStyle = ES_RIGHT;
	else dwStyle = ES_CENTER;

	rect.left += offset+4;
	rect.right = rect.left + GetColumnWidth( nCol ) - 3 ;
	if( rect.right > rcClient.right) rect.right = rcClient.right;

	dwStyle |= WS_BORDER|WS_CHILD|WS_VISIBLE|ES_AUTOHSCROLL;
	CEdit *pEdit = new CInPlaceEdit(nItem, nCol, GetItemText( nItem, nCol ));
	pEdit->Create( dwStyle, rect, this, IDC_IPEDIT);

	return pEdit;
}

void CEditableListCtrl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if( GetFocus() != this ) SetFocus();
	CSortableListCtrl::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CEditableListCtrl::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if( GetFocus() != this ) SetFocus();
	CSortableListCtrl::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CEditableListCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{	
	/*
	 * Function from: www.codeguru.com
	 */
	int index;
	CSortableListCtrl::OnLButtonDown(nFlags, point);

	int colnum;
	if( ( index = HitTestEx( point, &colnum )) != -1 )
	{
		UINT flag = LVIS_FOCUSED;
		if( (GetItemState( index, flag ) & flag) == flag && colnum > 0)
		{
			// Add check for LVS_EDITLABELS
			if( GetWindowLong(m_hWnd, GWL_STYLE) & LVS_EDITLABELS )
				EditSubLabel( index, colnum );
		}
		else
			SetItemState( index, LVIS_SELECTED | LVIS_FOCUSED ,
				    	LVIS_SELECTED | LVIS_FOCUSED); 
	}
}

BOOL CEditableListCtrl::PreTranslateMessage(MSG* pMsg)
{
	//source: http://forums.codeguru.com/showthread.php?242226-ctrl-A-Select-All-In-CListCtrl
	// hook to catch ctrl-a key for "select all"
	// and ctrl-i key for "invert selection
	
	if ( pMsg->message == WM_CHAR ){
		TCHAR chr = static_cast<TCHAR>(pMsg->wParam);
		switch ( chr ){
			case 0x01:	// 0x01 is the key code for ctrl-a and also for ctrl-A
				SelectAll();
				break;
			/*case 0x09:	// 0x09 is the key code for ctrl-i and ctrl-I
				InvertSelection();
				break; //*/
		}		
	}
	return CSortableListCtrl::PreTranslateMessage(pMsg);
}
