/*
    AQTRONIX C++ Library
    Copyright 2005-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// Whois.cpp: implementation of the CWhois class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Whois.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//define DEFAULTSERVER			_T("rs.internic.net")
//define DEFAULTSERVER			_T("whois.ripe.net")
#define DEFAULTSERVER			_T("whois.arin.net")
#define DEFAULTPORT				43
#define REFERRALSTARTKEY		_T("ReferralServer:")
#define REFERRALENDKEY			_T("\n")

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWhois::CWhois()
{
	AfxSocketInit();
	LoadDefaults();
}

CWhois::~CWhois()
{

}

inline void CWhois::LoadDefaults()
{
	Server = DEFAULTSERVER;
	Port = DEFAULTPORT;
}

CString CWhois::Lookup(LPCTSTR Domain, bool RecursiveLookup)
{
	return Lookup(Server,Domain,Port,NULL,NULL,RecursiveLookup);
}

CString CWhois::Lookup(LPCTSTR Server, LPCTSTR Domain, LPCTSTR Username, LPCTSTR Password, bool RecursiveLookup)
{
	return Lookup(Server,Domain,Port,Username,Password,RecursiveLookup);
}

CString CWhois::Lookup(LPCTSTR Server, LPCTSTR Domain, unsigned short Port, LPCTSTR Username, LPCTSTR Password, bool RecursiveLookup)
{
	try{

		CString Data(Domain);
		Data += "\r\n";
	
		CClientSocket cs;
		CString ret("");
		if(cs.CreateSync()){
			if(cs.Connect(Server,Port)){
				if(cs.Send(Data)){
					ret += cs.Receive();
				}else{
					ret += "Could not send query!";
				}
				cs.Close();
			}else{
				ret += "Could not connect to server!";
			}
		}
		//recursive lookups
		if(RecursiveLookup){
			int posstart = ret.Find(REFERRALSTARTKEY);
			if(posstart>-1){
				posstart += strlen(REFERRALSTARTKEY);
				int posend = ret.Find(REFERRALENDKEY,posstart);
				if(posend>-1){
					unsigned short PortRec = 0;
					CString ServerRec = CStringHelper::Trim(ret.Mid(posstart,posend-posstart));
					ServerRec.Replace("rwhois://",""); //ReferralServer:  rwhois://support.gorillaservers.com:4321
					ServerRec.Replace("whois://",""); //ReferralServer:  whois://whois.apnic.net
					if(ServerRec.Find(' ')==-1){//no space in url
						ret += "\r\n\r\n******* Performing recursive WHOIS lookup: " + ServerRec + " *******\r\n\r\n";
						//split server & port
						int possplit = ServerRec.Find(':');
						if(possplit>-1){
							//get port
							PortRec = CStringHelper::Safe_AsciiToInt(ServerRec.Mid(possplit+1),DEFAULTPORT,DEFAULTPORT);
							//get server
							ServerRec = ServerRec.Left(possplit);
						}
						//if no port, use default port
						if(PortRec<1){
							PortRec = DEFAULTPORT;
						}
						//do lookup
						ret += Lookup(ServerRec,Domain,PortRec,Username,Password);
					}
				}
			}
		}
		return ret;
			

	}catch(...){
		return _T("Fatal Error");
	}
}

/*
CString CWhois::Lookup(LPCTSTR Server, LPCTSTR Domain, unsigned short Port, LPCTSTR Username, LPCTSTR Password)
{
	try{

	debug.NewEntry(Server);
	debug.Append(Domain);
	debug.Append(CStringHelper::Safe_IntToAscii(Port));
	debug.Append(Username);
	debug.Append(Password);
	debug.Flush();
	
		CGopherConnection* connection = NULL;
		CString response("");
		try{
			debug.NewEntryFlush("try1");
			
			connection = Session.GetGopherConnection(Server,Username,Password,Port);
		}catch(CInternetException* pIE){
			debug.NewEntryFlush("catch1");

			connection = NULL;
			pIE->ReportError();
			pIE->Delete();
		}
		if(connection==NULL){
			response = "";
		}else{
			CGopherFile* gfile = NULL;
			try{
				debug.NewEntryFlush("try2");
				CGopherLocator locator = connection->CreateLocator(NULL,Domain,GOPHER_TYPE_TEXT_FILE);
				gfile = connection->OpenFile(locator);
			}catch(CInternetException* pIE){
				debug.NewEntryFlush("catch2");
				gfile = NULL;
				pIE->ReportError();
				pIE->Delete();
			}
			if(gfile==NULL){
				response = "";
			}else{
				
				CString line("");
				for (int i=0; i<20 && gfile->ReadString(line);i++){
					response += line;
					response += "\r\n";
				}
				if(gfile!=NULL)
					gfile->Close();

			}
			if(connection!=NULL)
				connection->Close();
		}
		return response;

	}catch(...){
		return "Fatal Error";
	}
}//*/

void CWhois::SetServer(LPCTSTR WhoisServer)
{
	Server = WhoisServer;
}

void CWhois::SetServer(LPCTSTR WhoisServer, unsigned short WhoisPort)
{
	Server = WhoisServer;
	Port = WhoisPort;
}
