#include "stdafx.h"
#include "LogAnalysisSettings.h"

CLogAnalysisSettings::CLogAnalysisSettings(void)
{
	LoadDefaults();
}

CLogAnalysisSettings::~CLogAnalysisSettings(void)
{
}

void CLogAnalysisSettings::LoadDefaults()
{
	General.EOL = _T("\r\n");
	General.Separator = "";
	General.Trim = false;
	General.FirstLineHeader = false;
	General.SortedColumn = -1;
	General.WordWrap = false;
	General.WhoisServer = _T("whois.arin.net");

	Filter.MaxEntries = 500000; //INT_MAX;
	Filter.Include.RemoveAll();
	Filter.Exclude.RemoveAll();
	Filter.Regex.RemoveAll();

	ExtractField.Enabled = false;
	ExtractField.Name = "";
	//ExtractField.Enabled = true;
	//ExtractField.Name = _T("User-Agent");

	//TODO: 2.0 - show country (flag) of IP address
	IPLookup.DNS = false;
	IPLookup.Country = false;
	IPLookup.ColumnIndex = 3;
}

bool CLogAnalysisSettings::WriteToFileINI(LPCTSTR fn)
{
	//Response handling
	//WRITE_INI_STRING(General,EOL)
	WRITE_INI_STRING(General,Separator)
	WRITE_INI_BOOL(General,Trim)
	WRITE_INI_BOOL(General,FirstLineHeader)
	WRITE_INI_INT(General,SortedColumn)
	WRITE_INI_BOOL(General,WordWrap)
	WRITE_INI_STRING(General,WhoisServer)
	WRITE_INI_INT(Filter,MaxEntries)
	WRITE_INI_CSTRINGLIST(Filter,Include)
	WRITE_INI_CSTRINGLIST(Filter,Exclude)
	//WRITE_INI_CSTRINGLIST(Filter,Regex) //TODO: FILTERS - regex

	return true;
}

bool CLogAnalysisSettings::ReadFromFileINI(LPCTSTR fn)
{
	const int sz = MAX_INI_FILE_SIZE;
	char buf[sz];
	DWORD l(0);

	//READ_INI_STRING(General,EOL)
	READ_INI_STRING(General,Separator)
	READ_INI_BOOL(General,Trim)
	READ_INI_BOOL(General,FirstLineHeader)
	READ_INI_INT(General,SortedColumn)
	READ_INI_BOOL(General,WordWrap)
	READ_INI_STRING(General,WhoisServer)
	READ_INI_INT(Filter,MaxEntries)
	READ_INI_CSTRINGLIST(Filter,Include,)
	READ_INI_CSTRINGLIST(Filter,Exclude,)
	READ_INI_CSTRINGLIST(Filter,Regex,)

	return true;
}

bool CLogAnalysisSettings::ParseXML(CString &key, CString &val)
{
	//if not a valid key
	if (key=="")
		return false;

	CString name = CXML::Decode(CXML::TagName(key));
	
	//Response Handling
	//	 PARSE_XML_STRING(General,EOL,"EOL")
	/*else*/ PARSE_XML_STRING(General,Separator,"Separator")
	else PARSE_XML_BOOL(General,Trim,"Trim")
	else PARSE_XML_BOOL(General,FirstLineHeader,"FirstLineHeader")
	else PARSE_XML_INT(General,SortedColumn,"SortedColumn",)
	else PARSE_XML_BOOL(General,WordWrap,"WordWrap")
	else PARSE_XML_STRING(General,WhoisServer,"WhoisServer")
	else PARSE_XML_INT(Filter,MaxEntries,"MaxEntries",)
	else PARSE_XML_CSTRINGLIST(Filter,Include,"Filter_Include")
	else PARSE_XML_CSTRINGLIST(Filter,Exclude,"Filter_Exclude")
	else PARSE_XML_CSTRINGLIST(Filter,Regex,"Filter_Regex")

	else{
		return false;
	}
	return true;
}

CString CLogAnalysisSettings::ToXML()
{
	CString xml("");
	CLogAnalysisSettings d;

	//WRITE_XML_STRING(General,EOL,"EOL","The End-Of-Line character or string")
	WRITE_XML_STRING(General,Separator,"Separator","The separator for the fields")
	WRITE_XML_BOOL(General,Trim,"Trim","Trim whitespaces and quotes")
	WRITE_XML_BOOL(General,FirstLineHeader,"FirstLineHeader","First line is the header")
	WRITE_XML_INT(General,SortedColumn,"SortedColumn","The column index to be sorted ascending")
	WRITE_XML_BOOL(General,WordWrap,"WordWrap","Word wrapping in the details view")
	WRITE_XML_STRING(General,WhoisServer,"WhoisServer","The WHOIS server to do lookups")
	WRITE_XML_INT(Filter,MaxEntries,"MaxEntries","The maximum number of entries shown in the list")
	WRITE_XML_CSTRINGLIST(Filter,Include,"Filter_Include","Only see entries with these strings")
	WRITE_XML_CSTRINGLIST(Filter,Exclude,"Filter_Exclude","Ignore entries with these strings")
	WRITE_XML_CSTRINGLIST(Filter,Regex,"Filter_Regex","Only see entries matching these regex patterns (syntax CAtlRegExp)")

	return xml;
}

bool CLogAnalysisSettings::WriteToFileXML(LPCTSTR fn)
{
	return WriteXMLEntityToFile(fn,"LogAnalysis",ToXML());
}

bool CLogAnalysisSettings::IsAllowedByFilters(CString& Line)
{
	//TODO: FILTERS - case-sensitive/insensitive "Match case" in UI
	if(Filter.Include.GetCount()>0){
		CStringList Matches;
		if(Filter.Include.GetCount()!=CStringHelper::CountKeywords(Line,Filter.Include,Matches)){
			return false;
		}
	}
	if(Filter.Exclude.GetCount()>0){
		if(CStringHelper::IsListItemInStringCompareNoCase(Line,Filter.Exclude)){
			return false;
		}
	}

#ifdef REGEX
	if(Filter.Regex.GetCount()>0){
		POSITION pos = Filter.Regex.GetHeadPosition();
		CString pattern;
		while(pos!=NULL){
			pattern = Filter.Regex.GetNext(pos);
			CAtlRegExp<>* regex = regex_cache.Lookup(pattern);
			if(regex!=NULL){ //NULL = regex syntax error
				CAtlREMatchContext<> mc;
				if(!regex->Match(Line,&mc)){
					return false;
				}
			}
		}
	}
#endif

	return true;
}

