/*
    AQTRONIX C++ Library
    Copyright 2016 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
#include "stdafx.h"
#include "LogReader.h"

CLogReader::CLogReader(void):IsReading(false),AbortReading(false),filename(""),CurrentPosition(0),ColumnCount(0),HeaderName(""),HeaderDate("")
{
}

CLogReader::~CLogReader(void)
{
}

bool CLogReader::ReadLog(LPCTSTR FileName, LONGLONG Start)
{
	try{
		IsReading = true;
		AbortReading = false;

		filename = FileName;
		CString line("");
		static int const bufsize = 32767;
		char bufread[bufsize+1]; //+1 for termination
		//read log
		CFile flog;
		if (!flog.Open(filename,CFile::modeRead|CFile::shareDenyNone,NULL)){
			return false;
		}else{
			CStringList lines;
			UINT size(0);
			UINT eollength = Settings.General.EOL.GetLength();

			//read first chunk for headers or move to certain position
			if(Start>0){
				//TODO: PARSING - BUG: problem with last line not ending with CRLF between ReadLogAppend calls (one log entry over 2 lines)
				flog.Seek(Start,CFile::begin);
			}else{
				bool firstline = true;
				size = flog.Read(bufread,bufsize);
				if(size>0){
					bufread[size] = '\0';
					POSITION pos;
					CStringList Headers;
					//detect EOL
					Settings.General.EOL = Settings.DetectEOL(bufread);
					eollength = Settings.General.EOL.GetLength();

					CStringHelper::Split(Headers,bufread,Settings.General.EOL);
					pos = Headers.GetHeadPosition();
					while(pos!=NULL){
						line = Headers.GetNext(pos);
						if (line.Left(strlen(METALINE))==METALINE){
							SetHeaders(line); //parse headers (columns,...)
							firstline = false;
						}else{
							//first log entry are column names
							if(firstline && Settings.General.FirstLineHeader){
								//if(Settings.General.Separator=="")
								//	Settings.General.Separator = Settings.DetectSeparator(line);  //don't do this: unable to manually select something else
								SetColumns(line,true);
								firstline = false;
							}else{
								//log entries
								lines.AddTail(line);
							}
						}
					}
					//add CRLF if those were the last 2 chars (removed by Split())
					if(size>eollength){
						CString endline(_T(""));
						for(UINT i = size-eollength; i < size; i++){
							endline += bufread[i];
						}
						if(endline == Settings.General.EOL){
							line += Settings.General.EOL;
						}
					}

					//add columns by manually set separator if none defined in log file
					if(ColumnCount==0 && Settings.General.Separator.GetLength()>0){
						POSITION pos = lines.GetHeadPosition();
						if(pos!=NULL){
							SetColumns(lines.GetNext(pos),false);
						}
					}

					//add a single column if there are none
					if(ColumnCount==0){
						SetSingleColumn();
					}

					//remove last read incomplete line, will be added later
					if(!lines.IsEmpty())
						lines.RemoveTail();

					//add log entries
					AddLines(lines);
				}
			}
					
			//read rest of file
			size = flog.Read(bufread,bufsize);
			while (size>0 && !AbortReading){
				bufread[size] = '\0'; //make sure it is null terminated
				CStringHelper::Split(lines,line + CString(bufread),Settings.General.EOL);
				line = lines.GetTail();
				//add CRLF if that were the last 2 chars (removed by Split())
				if(size>eollength){
					CString endline(_T(""));
					for(UINT i = size-eollength; i < size; i++){
						endline += bufread[i];
					}
					if(endline == Settings.General.EOL){
						line += Settings.General.EOL;
					}
				}
				if(!lines.IsEmpty())
					lines.RemoveTail();
				AddLines(lines);
				size = flog.Read(bufread,bufsize);
			}

			//last line
			if(line!="" && !AbortReading){
				//remove last CRLF as nothing more will be read
				if(line.Right(eollength)==Settings.General.EOL){
					line = line.Left(line.GetLength()-eollength);
				}
				AddLine(line);
			}

			//remember position and close
			CurrentPosition = (ULONGLONG)flog.GetPosition();
			flog.Close();
		}
		IsReading = false;
		return true;
	
	}catch(...){
		IsReading = false;
		return false;
	}
}

bool CLogReader::ReadLogAppend()
{
	if(filename.GetLength()==0 || IsReading){
		return false;
	}else{
		return ReadLog(filename,CurrentPosition);
	}
}

bool CLogReader::HasFile()
{
	return filename.GetLength()>0;
}

void CLogReader::SetHeaders(CString& Header)
{
	try{

	//#Fields:
	if (Header.Left(strlen(HEADERFIELD))==HEADERFIELD && FieldList.GetSize()==0){
		Settings.General.Separator = DetectFieldSeparator(Header);
		SetColumns(Header.Mid(strlen(HEADERFIELD)),true);
	}else{
		if(Header.Left(strlen(HEADERSOFTWARE))==HEADERSOFTWARE){
			HeaderName = Header.Mid(strlen(HEADERSOFTWARE));
		}else{
			if(Header.Left(strlen(HEADERDATE))==HEADERDATE){
				HeaderDate = Header.Mid(strlen(HEADERDATE));
			}
		}
	}

	}catch(...){
		//ignore error
	}
}

void CLogReader::SetColumns(CString& Fields, bool UseFieldNameAsColumnHeading)
{
	CStringList Flds;
	CStringHelper::Split(Flds,Fields,Settings.General.Separator);
	if(Settings.General.Trim){
		POSITION pos = Flds.GetHeadPosition();
		while(pos!=NULL){
			CString& fld = Flds.GetNext(pos);
			fld = TrimField(fld);
		}
	}
	SetColumns(Flds,UseFieldNameAsColumnHeading);
}

void CLogReader::AddLines(CStringList &lines)
{
	POSITION pos = lines.GetHeadPosition();
	while(pos!=NULL){
		AddLine(lines.GetNext(pos));
	}
	lines.RemoveAll();
}

CString& CLogReader::TrimField(CString& field)
{
	#if _MSC_VER > 1200
		return field.Trim().Trim('"');
	#else
		return CStringHelper::Trim(CStringHelper::Trim(field),'"');
	#endif
}

CString CLogReader::DetectFieldSeparator(CString& Line)
{
	const int total = 8;
	CString list[] = {
		_T(" ; "),
		_T("\t"),
		_T("\";\""),	//";"
		_T("\",\""),	//","
		_T("\" \""),	//" "
		_T(";"),
		_T(","),
		_T(" "),
	};

	for(int i=0; i<total; i++){
		if(Line.Find(list[i],0)!=-1)
			return list[i];
	}

	return _T(""); //if no header separator found, use complete line as header
}
