#include "stdafx.h"
#include "LogViewList.h"
#include "StringHelper.h"
#include "MainFrm.h"
#include "IPInfo.h"

IMPLEMENT_DYNCREATE(CLogViewList, CView)

BEGIN_MESSAGE_MAP(CLogViewList, CView)
	ON_WM_CREATE()
	ON_WM_SIZE()
END_MESSAGE_MAP()

CLogViewList::CLogViewList(void):filtered_count(0)
{
}

CLogViewList::~CLogViewList(void)
{
}

BOOL CLogViewList::PreCreateWindow(CREATESTRUCT& cs)
{
	// Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CLogViewList drawing

void CLogViewList::OnDraw(CDC* /*pDC*/)
{
	//CsplitterbasicDoc* pDoc = GetDocument();
	//ASSERT_VALID(pDoc);
	//if (!pDoc)
	//	return;

	// add draw code for native data here
}


// CLogViewList printing

void CLogViewList::PrintLog()
{ 
	this->OnFilePrint();
}

void CLogViewList::PrintPreview()
{ 
	this->OnFilePrintPreview();
}

BOOL CLogViewList::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CLogViewList::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// TODO: add extra initialization before printing
}

void CLogViewList::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

bool CLogViewList::Export(LPCTSTR FileName)
{
	CStdioFile f;
	if (f.Open(FileName,CFile::modeCreate|CFile::modeWrite|CFile::shareDenyWrite|CFile::typeBinary,NULL)){
		//headers
		CString headers = HEADERSOFTWARE;
		headers += "LogAnalysis export of " + filename + " (" + this->HeaderName + ")" + Settings.General.EOL;
		headers += HEADERFIELD;
		headers	+= CStringHelper::ToString(this->FieldList,Settings.General.Separator);
		headers += Settings.General.EOL;
		headers += HEADERDATE;
		headers += this->HeaderDate;
		headers += Settings.General.EOL;
		f.WriteString(headers);

		//entries
		int nCount = loglist.GetItemCount();
		int subindex;
		CString Row;
		for (int x = 0; x < nCount; x++)
		{
			//get subitems
			subindex = 0;
			Row = "";
			while(subindex<ColumnCount-1){
				Row += loglist.GetItemText(x, subindex);
				Row += Settings.General.Separator;
				subindex++;
			}
			Row += loglist.GetItemText(x, subindex);
			Row += Settings.General.EOL;

			f.WriteString(Row);
		}
		
		f.Close();
		return true;
	}
	return false;
}

inline void CLogViewList::Clear()
{
	FieldList.RemoveAll();
	loglist.DeleteAllItems();			//delete all rows
	while(loglist.DeleteColumn(0)!=0);	//delete all columns
	//filename = ""; don't clear this: see RefreshLog();
	CurrentPosition = 0;
	ColumnCount = 0;
	HeaderName = "";
	HeaderDate = "";
	//Separator = "";
	filtered_count = 0;
}

void CLogViewList::CloseLog()
{
	Clear();
	filename="";
}

void CLogViewList::RefreshLog()
{
	Clear();
	if(filename!="")
		ReadLog(filename);
}

void CLogViewList::SetColumns(CStringList &Fields, bool UseFieldNameAsColumnHeading)
{
	ColumnCount = 0;
	FieldList.RemoveAll();
	POSITION pos;
	CString fld;
	pos = Fields.GetTailPosition();
	while(pos!=NULL){
		fld = Fields.GetPrev(pos);
		loglist.InsertColumn(0,UseFieldNameAsColumnHeading?fld:"",LVCFMT_LEFT,CalculateColumnWidth(fld,70,500));
		ColumnCount++;
	}

	//internal headerlist
	if(UseFieldNameAsColumnHeading){
		FieldList.SetSize(ColumnCount);
		pos = Fields.GetHeadPosition();
		int i=0;
		while(pos!=NULL){
			FieldList.SetAt(i,Fields.GetNext(pos));
			i++;
		}
	}

	if(Settings.ExtractField.Enabled)
		loglist.InsertColumn(ColumnCount,CString(Settings.ExtractField.Name),LVCFMT_LEFT,200);

}

void CLogViewList::SetSingleColumn()
{
	CString s; s.LoadString(IDS_APP_LOG_ENTRIES);
	loglist.InsertColumn(0,s,LVCFMT_LEFT,600);
	ColumnCount++;
}

int CLogViewList::CalculateColumnWidth(CString& Fld, int MinimumLength, int MaximumLength)
{
	const int FONTWIDTH_ = 10;
	int Length = Fld.GetLength();
	if(Length>MaximumLength/FONTWIDTH_){
		return MaximumLength;
	}else{
		if(Length<MinimumLength/FONTWIDTH_){
			return MinimumLength;
		}else{
			return Length*FONTWIDTH_;
		}
	}
}

bool CLogViewList::AddLine(CString& Line)
{
	try{
		if(Line.Left(strlen(METALINE))==METALINE || AbortReading){
			return false;
		}else{
			int lnumber = loglist.GetItemCount();
			if(Settings.IsAllowedByFilters(Line) && !(Settings.Filter.MaxEntries > 0 && lnumber == Settings.Filter.MaxEntries) ){
				int i(0);
				POSITION pos;
				CStringList FieldItems;
				CStringHelper::Split(FieldItems,Line,Settings.General.Separator,ColumnCount);
				lnumber = loglist.InsertItem(lnumber+1,Line);
				if(lnumber==-1){
					return false;
				}else{
					SetEntryProgress(lnumber+1);
					if(AbortReading)
						return false;

					pos = FieldItems.GetHeadPosition();
					if(Settings.General.Trim){
						while (pos!=NULL){
							CString Item = TrimField(FieldItems.GetNext(pos));;
							loglist.SetItem(lnumber,i,LVIF_TEXT,Item,0,0,0,0);
							i++;
						}
					}else{
						while (pos!=NULL){
							CString Item = FieldItems.GetNext(pos);
							loglist.SetItem(lnumber,i,LVIF_TEXT,Item,0,0,0,0);
							i++;
						}
					}

					if(Settings.ExtractField.Enabled){
						CString Extracted = ExtractField(Line, Settings.ExtractField.Name + ": '","'");
						loglist.SetItem(lnumber,i,LVIF_TEXT,Extracted,0,0,0,0);
					}

					return true;
				}
			}else{
				filtered_count++;
				return false;
			}
		}
	
	}catch(...){
		return false;
	}
}

CString CLogViewList::ExtractField(CString& Line, CString MatchStart, CString MatchEnd)
{
	//TODO: Extract field - regex
	int posstart = Line.Find(MatchStart);
	if(posstart>-1){
		posstart = posstart + MatchStart.GetLength();
		if(posstart<Line.GetLength()-1){
			int posend = Line.Find(MatchEnd,posstart);
			if(posend>-1 && posend>posstart)
				return Line.Mid(posstart,posend-posstart);
		}
	}
	return CString("");
}

void CLogViewList::BindDetailToList()
{
	CString Row = GetCurrentEntry();
	if(Row.GetLength()>0){
		pDetails->SetDetailView(Row,Settings.General.Separator,ColumnCount);

		if(Settings.IPLookup.DNS || Settings.IPLookup.Country){
			POSITION pos = loglist.GetFirstSelectedItemPosition();
			if (pos != NULL){
				int nItem =  loglist.GetNextSelectedItem(pos);
				CString ip = loglist.GetItemText(nItem,Settings.IPLookup.ColumnIndex);
				if(ip.GetLength()>0){
					static CIPInfo ipinfo;

					CMainFrame *pMainWnd = (CMainFrame *)AfxGetMainWnd();

					//DNS lookup
					if(Settings.IPLookup.DNS){
						pMainWnd->SetStatusBar(ipinfo.DNSLookup(ip));
					}

					//Country lookup
					if(Settings.IPLookup.Country){
						pMainWnd->SetStatusBar(ipinfo.LookupCountry(ip));
					}
				}
			}
		}
	}
}

CString CLogViewList::GetCurrentEntry()
{
	CString Row("");
	if(!IsReading){
		static const int bufsize = 128000;  //HACK: not thread safe + size?
		static char bufbind[bufsize];	

		POSITION pos;
		LVITEM item;
		item.mask = LVIF_TEXT;
		item.pszText = bufbind;
		item.cchTextMax = bufsize;
		int subindex(0);
		CString labelseparator(": ");
		labelseparator.Replace(Settings.General.Separator,"");//remove separator, otherwise it will split
		//get item
		pos = loglist.GetFirstSelectedItemPosition();
		if(pos!=NULL){
			item.iItem = loglist.GetNextSelectedItem(pos);
			//get subitems
			while(subindex<ColumnCount){
				item.iSubItem = subindex;
				if(loglist.GetItem(&item)){
					if(subindex<FieldList.GetSize()){ //get fieldlabels if there are any
						Row += FieldList.GetAt(subindex);
						Row += labelseparator;
					}
					Row += item.pszText;
					Row += Settings.General.Separator;
				}
				subindex++;
			}
		}
	}
	return Row;
}

int CLogViewList::EntryCount() const
{
	return loglist.GetItemCount();
}

int CLogViewList::FilteredCount() const
{
	return filtered_count;
}

void CLogViewList::SetEntryProgress(int number)
{
	static CMainFrame* pmf = NULL;
	if(pmf==NULL)
		pmf=(CMainFrame*)AfxGetApp()->m_pMainWnd;

	pmf->SetProgress(number);
}

void CLogViewList::AddFilter(CString& Text, int FilterType)
{
	if(Text.GetLength()>0){
		switch(FilterType){
			case 0:
				Settings.Filter.Include.AddTail(Text);
				break;
			case 1:
				Settings.Filter.Exclude.AddTail(Text);
				break;
		}
		RefreshLog();
	}
}

void CLogViewList::ClearFilters()
{
	Settings.Filter.Include.RemoveAll();
	Settings.Filter.Exclude.RemoveAll();
	Settings.Filter.Regex.RemoveAll();
	RefreshLog();
}

int CLogViewList::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	//Add your specialized creation code here

	//set font
	LOGFONT logfont;
	logfont.lfHeight = 14;
	logfont.lfWidth = 0;
	logfont.lfEscapement = 0;
	logfont.lfOrientation = 0;
	logfont.lfWeight = FW_NORMAL;
	logfont.lfItalic = FALSE;
	logfont.lfUnderline = FALSE;
	logfont.lfStrikeOut = FALSE;
	logfont.lfCharSet = ANSI_CHARSET;
	logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	logfont.lfQuality = DEFAULT_QUALITY;
	logfont.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(logfont.lfFaceName,"Verdana");
	font.CreateFontIndirect(&logfont);

	CRect cr;
	GetClientRect(&cr);
	if (!loglist.Create(WS_CHILD|WS_VISIBLE|WS_TABSTOP|WS_BORDER|LVS_REPORT|LVS_SINGLESEL|LVS_SHOWSELALWAYS,cr,this,ID_LIST)){
		return -1;
	}else{
		loglist.SetFont(&font);
		loglist.SetExtendedStyle(LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT);
	}

	return 0;
}

void CLogViewList::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);

	//Add your message handler code here
	CRect cr;
	GetClientRect(&cr);
	loglist.MoveWindow(&cr);
}

BOOL CLogViewList::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	//Add your specialized code here and/or call the base class
	if(wParam==ID_LIST){

		LPNMHDR lpnmhdr;

#if (_WIN32_IE >= 0x0400)
		//version 4.71 ->
		LPNMITEMACTIVATE lpnmia;
		lpnmia = (LPNMITEMACTIVATE) lParam;
		lpnmhdr = (LPNMHDR) (&(lpnmia->hdr));
#else
		//version 0 -> 4.70
	    lpnmhdr = (LPNMHDR) lParam;
#endif

		if(lpnmhdr->code==NM_CLICK){
			ReadLogAppend();	//Refresh
		}
		if(lpnmhdr->code==LVN_KEYDOWN){
			LPNMLVKEYDOWN pnkd = (LPNMLVKEYDOWN) lParam;
			if(pnkd->wVKey == VK_F5){
				ReadLogAppend();	//Refresh
			}
			if(pnkd->wVKey == VK_SPACE){ //spacebar is same as arrow down
				loglist.MoveDown();
			}
			if(pnkd->wVKey == VK_BACK){ //backspace is same as arrow up
				loglist.MoveUp();
			}
		}
		if(lpnmhdr->code==LVN_ITEMACTIVATE || lpnmhdr->code==LVN_ITEMCHANGED){
			/* if(!IsReading){
				ReadLogAppend();	//Refresh
			} //*/
			BindDetailToList();
		}
	}
	return CView::OnNotify(wParam, lParam, pResult);
}

void CLogViewList::SortColumn(int ColumnIndex, bool Ascending)
{
	loglist.SortColumn(ColumnIndex,Ascending);
}