// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "LogAnalysis.h"

#include "MainFrm.h"

#include "StringHelper.h"
#include "FileName.h"
#include "Clipboard.h"
#include "DialogLogfiles.h"
#include "DialogWhois.h"
#include "DialogDNS.h"
#include "DialogFilter.h"
#include "DialogFilterEdit.h"
#include "DialogSeparator.h"
#include "DialogMaxEntries.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_CLOSE, OnFileClose)
	ON_COMMAND(ID_OPEN, OnFileOpen)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_COMMAND(ID_EDIT_WHOISLOOKUP, OnWhoisLookup)
	ON_COMMAND(ID_EDIT_DNSLOOKUP, OnDNSLookup)
	ON_COMMAND(ID_LIST_DOWN, OnListDown)
	ON_COMMAND(ID_LIST_UP, OnListUp)
	ON_COMMAND(ID_VIEW_SAVE, OnSaveSettings)
	ON_COMMAND(ID_VIEW_EXPORT, OnExportView)
	ON_COMMAND(ID_VIEW_SEPARATOR, OnSetSeparator)
	ON_COMMAND(ID_VIEW_TRIM, OnSetTrim)
	ON_COMMAND(ID_VIEW_FIRSTLINEHEADER, OnSetFirstLineHeader)
	ON_COMMAND(ID_VIEW_WORDWRAP, OnSetWordWrap)
	ON_COMMAND(ID_FILTER_ADD, CMainFrame::OnFilterAdd)
	ON_COMMAND(ID_FILTER_EDIT, CMainFrame::OnFilterEdit)
	ON_COMMAND(ID_FILTER_MAXENTRIES, CMainFrame::OnFilterMaxEntries)
	ON_COMMAND(ID_FILTER_CLEAR, CMainFrame::OnFilterClear)
	ON_COMMAND(ID_FILE_PRINT, CMainFrame::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CMainFrame::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CMainFrame::OnFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateCanUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateCanRedo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateNeedSelection)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateNeedSelection)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateNeedClip)
	ON_UPDATE_COMMAND_UI(ID_CLOSE, OnUpdateNeedFile)
	ON_UPDATE_COMMAND_UI(ID_OPEN, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateNeedFile)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_DIRECT, OnUpdateNeedFile)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateNeedFile)
	ON_UPDATE_COMMAND_UI(ID_FILTER_ADD, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_FILTER_EDIT, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_FILTER_CLEAR, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_FILTER_MAXENTRIES, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_VIEW_EXPORT, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FIRSTLINEHEADER, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SEPARATOR, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TRIM, OnUpdateNotReading)
	ON_UPDATE_COMMAND_UI(ID_DETAILS_FILTER, OnUpdateNotReading)
	//}}AFX_MSG_MAP
	ON_WM_CLOSE()
	ON_WM_TIMER()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_SEPARATOR,           // log entries
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

#define IDT_TIMER_DEFERRED WM_USER + 200

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	AbortOpenFile = false;
	DeferredFile.Reset();
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	//old view
	// create a view to occupy the client area of the frame
	/*if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW,
		CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}*/
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	//Delete these three lines if you don't want the toolbar to
	//be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	//default settings in startup folder
	this->pList->Settings.ReadFromFile(".\\LogAnalysis.ini");
	SyncMenu();

	//load log file
	LoadDefaultFile();
	SetTimer(IDT_TIMER_DEFERRED, 100, NULL);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	//Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::OnSetFocus(CWnd* pOldWnd)
{
	//old view
	// forward focus to the view window
	//m_wndView.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndSplitter.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CMainFrame::LoadDefaultFile()
{
	//if parameter given, open that file
	CString fn(AfxGetApp()->m_lpCmdLine);
	if(!fn.IsEmpty()){
		#if _MSC_VER > 1200
			fn = fn.Trim('"'); //for drag&drop on window
		#else
			fn = CStringHelper::Trim(fn,'"'); //for drag&drop on window
		#endif
		DeferredFile.Name = fn;
	}else{
		/*
		 * Loads a default file if present, if file not found a message
		 * will be displayed. If more than one a listbox will be shown
		 */
		CDialogLogfiles dlg;
		if (dlg.DoModal()==IDOK){
			DeferredFile.Name = dlg.LogDir + dlg.m_ListLogFilesText;
			DeferredFile.Title = dlg.m_ListLogFilesText;
		}
	}
}

void CMainFrame::LoadDeferredFile()
{
	if(!DeferredFile.IsEmpty()){
		if(OpenFile(DeferredFile.Name)){
			if(DeferredFile.Title.GetLength()>0)
				SetWindowTitle(DeferredFile.Title);
			else
				SetWindowTitle(DeferredFile.Name);

			SetStatusBar(pList->HeaderName + " - " + pList->HeaderDate);
			SetStatusBarCount(); //need this: no paint event
		}
		DeferredFile.Reset();
	}
}

void CMainFrame::SetWindowTitle(LPCTSTR Caption)
{
	CString s; s.LoadString(AFX_IDS_APP_TITLE);
	SetWindowText(s + " - " + Caption);
}

void CMainFrame::SetStatusBar(LPCTSTR Text)
{
	m_wndStatusBar.SetPaneText(0,Text);
}

void CMainFrame::SetStatusBarCount()
{
	CString s; s.LoadString(IDS_APP_LOG_ENTRIES);
	CString f; f.LoadString(IDS_APP_FILTERED_ENTRIES);
	s = s + ": " + CStringHelper::Safe_IntToAscii(pList->EntryCount()) + " - " + f + ": " + CStringHelper::Safe_IntToAscii(pList->FilteredCount());
	m_wndStatusBar.SetPaneText(1,s);
}

void CMainFrame::SetProgress(long l)
{
	CString s; s.LoadString(IDS_APP_PROCESSING);
	SetProgress(s + " " + CStringHelper::Safe_LongToAscii(l));
}

void CMainFrame::SetProgress(LPCTSTR message)
{
	static CTime lastchecked = CTime::GetCurrentTime();

	SetStatusBar(message);

	CTime now = CTime::GetCurrentTime();
	if(now-lastchecked>CTimeSpan(1)){
		DoEvents();
	}
}

void CMainFrame::DoEvents()
{
	//source:
	//http://stackoverflow.com/questions/1415095/doevents-equivalent-for-c
	//https://msdn.microsoft.com/en-us/library/windows/desktop/ms644928%28v=vs.85%29.aspx

	MSG msg;
    BOOL result;

    while ( ::PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE ) )
    {
        result = ::GetMessage(&msg, NULL, 0, 0);
        if (result == 0) // WM_QUIT
        {
            ::PostQuitMessage(msg.wParam);
            break;
        }
        else if (result == -1)
        {
             // Handle errors/exit application, etc.
        }
        else 
        {
			if(!TranslateAccelerator(this->m_hWnd,this->m_hAccelTable,&msg)){ //for accelerator keys to work while reading log file
				::TranslateMessage(&msg);
				::DispatchMessage(&msg);
			}
        }
    }
}

void CMainFrame::OnFileOpen()
{
	CString filter;
	filter.LoadString(IDS_APP_LOGFILES);
	filter += " (*.log)|*.log|";
	CString s;
	s.LoadString(IDS_APP_ALLFILES);
	filter += s;
	filter += " (*.*)|*.*||";
	CFileDialog fd(TRUE,NULL,NULL,OFN_FILEMUSTEXIST,filter,NULL);
	if(fd.DoModal()==IDOK){
		CString fn = fd.GetPathName();

		//read log file
		if(OpenFile(fn)){
			SetWindowTitle(fn);
			SetStatusBar(pList->HeaderName + " - " + pList->HeaderDate);
			SetStatusBarCount(); //need this: no paint event
		}
	}	
}

void CMainFrame::OnFileClose()
{
	pList->AbortReading = true;
	this->AbortOpenFile = true;
	Clear();
	CString s; s.LoadString(AFX_IDS_APP_TITLE);
	SetWindowText(s);
}

void CMainFrame::OnEditUndo()
{
	this->pDetails->Undo();
}

void CMainFrame::OnEditRedo()
{
	this->pDetails->Redo();
}

void CMainFrame::OnEditCut()
{
	this->pDetails->Cut();
}

void CMainFrame::OnEditCopy()
{
	this->pDetails->Copy();
}

void CMainFrame::OnEditPaste()
{
	this->pDetails->Paste();
}

//http://support.microsoft.com/kb/145616
void CMainFrame::OnUpdateNeedSelection(CCmdUI* pCmdUI)
{
     // get the current focus & determine if its on a CEdit control
      CWnd* pWnd = GetFocus();
	  if (NULL == pWnd || !(pWnd->m_hWnd == this->pDetails->details.m_hWnd))
      {
         pCmdUI->Enable( FALSE );
      }
      else
      {
         CEdit* pEdit = (CEdit*)pWnd;
         int nBeg, nEnd;

         pEdit->GetSel( nBeg, nEnd );
         pCmdUI->Enable( nBeg != nEnd );
      }
}

void CMainFrame::OnUpdateNeedClip(CCmdUI* pCmdUI)
{
      // get the current focus & determine if its on a CEdit control
      // also check to see if the control is read-only.
      CWnd* pWnd = GetFocus();
      if (  NULL == pWnd ||
            !(pWnd->m_hWnd == this->pDetails->details.m_hWnd) ||
            (pWnd->GetStyle() & ES_READONLY) != 0 )
      {
         pCmdUI->Enable( FALSE );
      }
      else
         pCmdUI->Enable(::IsClipboardFormatAvailable(CF_TEXT));
}

void CMainFrame::OnUpdateNeedFile(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(this->pList->HasFile());
}

void CMainFrame::OnUpdateNotReading(CCmdUI *pCmdUI)
{
	pCmdUI->Enable(!this->pList->IsReading);
}

void CMainFrame::OnUpdateCanUndo(CCmdUI* pCmdUI)
{
     // get the current focus & determine if its on a CEdit control
      CWnd* pWnd = GetFocus();
	  if (NULL == pWnd || !(pWnd->m_hWnd == this->pDetails->details.m_hWnd))
      {
         pCmdUI->Enable( FALSE );
      }
      else
      {
         pCmdUI->Enable( this->pDetails->CanUndo() );
      }
}

void CMainFrame::OnUpdateCanRedo(CCmdUI* pCmdUI)
{
     // get the current focus & determine if its on a CEdit control
      CWnd* pWnd = GetFocus();
	  if (NULL == pWnd || !(pWnd->m_hWnd == this->pDetails->details.m_hWnd))
      {
         pCmdUI->Enable( FALSE );
      }
      else
      {
         pCmdUI->Enable( this->pDetails->CanRedo() );
      }
}

void CMainFrame::OnListDown()
{
	this->pList->loglist.MoveDown();
}

void CMainFrame::OnListUp()
{
	this->pList->loglist.MoveUp();
}

void CMainFrame::OnSaveSettings()
{
	this->pList->Settings.General.SortedColumn = this->pList->loglist.SortedColumn;

	//save settings
	this->pList->Settings.WriteToFile(".\\LogAnalysis.ini");

	//show message in status bar
	CString s; s.LoadString(IDS_APP_SAVED);
	s += "LogAnalysis.ini";
	this->SetStatusBar(s);
}

void CMainFrame::OnExportView()
{
	CString filter;
	filter.LoadString(IDS_APP_LOGFILES);
	filter += " (*.log)|*.log|";
	CString s;
	s.LoadString(IDS_APP_ALLFILES);
	filter += s;
	filter += " (*.*)|*.*||";
	CFileDialog fd(FALSE,".log","Export.log",OFN_OVERWRITEPROMPT,filter,NULL);
	if(fd.DoModal()==IDOK){
		CString fn = fd.GetPathName();

		//export filtered view to file
		if(this->pList->Export(fn)){

			//show message in status bar
			CString s; s.LoadString(IDS_APP_SAVED);
			s += fn;
			this->SetStatusBar(s);
		}
	}
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
	//splitter
	CRect cr;
	GetClientRect(&cr);

	CSize paneSize(cr.Width(), cr.Height() / 2);

	int rc;
	m_wndSplitter.CreateStatic(this, 2, 1);

	//list
	rc = m_wndSplitter.CreateView(0, 0,RUNTIME_CLASS(CLogViewList), paneSize, pContext);
	if (FALSE == rc)
		return rc;

	//details
	rc = m_wndSplitter.CreateView(1, 0, RUNTIME_CLASS(CLogViewDetails), paneSize, pContext);
	this->pList = (CLogViewList*) m_wndSplitter.GetPane(0,0);
	this->pDetails = (CLogViewDetails*) m_wndSplitter.GetPane(1,0);

	//link list to details
	pList->pDetails = pDetails;

	return rc;
}

BOOL CMainFrame::PreTranslateMessage(MSG* pMsg) 
{
	//show popup menu for details
	if(pMsg->hwnd==pDetails->details.m_hWnd){
		if (pMsg->message==WM_RBUTTONDOWN){
			CMenu menu;
			DWORD dwSelected;
			menu.LoadMenu(IDR_MENU_DETAILS);
			CMenu *pmenu = menu.GetSubMenu(0);
			dwSelected = pmenu->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_NONOTIFY|TPM_RETURNCMD,pMsg->pt.x,pMsg->pt.y,this,NULL);
			menu.DestroyMenu();
			if(dwSelected>0){
				CString selection = pDetails->GetSelText();
				switch(dwSelected){
				case ID_DETAILS_COPY:
					SetClipboardText(selection);
					break;
				case ID_DETAILS_FILTER:
					AddFilter(selection);
					break;
				case ID_DETAILS_WHOIS:
					WhoisLookup(selection);
					break;
				case ID_DETAILS_DNSLOOKUP:
					DNSLookup(selection);
					break;
				default:
					CString s;
					s.LoadString(IDS_APP_UNKNOWNCOMMAND);
					AfxMessageBox(s);
				}
			}
			return TRUE;
		}
	}else if(pMsg->hwnd==pList->loglist.m_hWnd){
		//TRACE("MESSAGE: %i\n",pMsg->message);
		if (pMsg->message==WM_PAINT)
			SetStatusBarCount();
	}

	return CFrameWnd::PreTranslateMessage(pMsg);
}

void CMainFrame::OnSetSeparator()
{
	SetSeparator();
}

void CMainFrame::OnSetTrim()
{
	this->pList->Settings.General.Trim = !this->pList->Settings.General.Trim;
	SetMenuItemCheck(2,ID_VIEW_TRIM,this->pList->Settings.General.Trim);
	this->pList->RefreshLog();
	this->pDetails->Clear();
}

void CMainFrame::OnSetFirstLineHeader()
{
	this->pList->Settings.General.FirstLineHeader = !this->pList->Settings.General.FirstLineHeader;
	SetMenuItemCheck(2,ID_VIEW_FIRSTLINEHEADER,this->pList->Settings.General.FirstLineHeader);
	this->pList->RefreshLog();
	this->pDetails->Clear();
}

void CMainFrame::OnSetWordWrap()
{
	this->pList->Settings.General.WordWrap = !this->pList->Settings.General.WordWrap;
	SetMenuItemCheck(2,ID_VIEW_WORDWRAP,this->pList->Settings.General.WordWrap);
	this->pDetails->WordWrapping(this->pList->Settings.General.WordWrap);
}

void CMainFrame::SetMenuItemCheck(int submenu, UINT ID, bool checked)
{
	CMenu* pMenu = this->GetMenu();
	CMenu* pSubMenu = pMenu->GetSubMenu(submenu);
	if(checked){
		pSubMenu->CheckMenuItem(ID,MF_CHECKED|MF_BYCOMMAND);
	}else{
		pSubMenu->CheckMenuItem(ID,MF_UNCHECKED|MF_BYCOMMAND);
	}
}

void CMainFrame::SyncMenu()
{
	SetMenuItemCheck(2,ID_VIEW_TRIM,this->pList->Settings.General.Trim);
	SetMenuItemCheck(2,ID_VIEW_FIRSTLINEHEADER,this->pList->Settings.General.FirstLineHeader);
	SetMenuItemCheck(2,ID_VIEW_WORDWRAP,this->pList->Settings.General.WordWrap);
	if(this->pList->Settings.General.WordWrap)
		this->pDetails->WordWrapping(true);
}

void CMainFrame::OnFilterAdd()
{
	AddFilter(pDetails->GetSelText());
}

void CMainFrame::OnFilterEdit()
{
	//TODO: FILTERS - Regex in CDialogFilterEdit
	CDialogFilterEdit dlg;
	dlg.pInclude = &this->pList->Settings.Filter.Include;
	dlg.pExclude = &this->pList->Settings.Filter.Exclude;
	if(dlg.DoModal()==IDOK){
		this->pList->RefreshLog();
		this->pDetails->Clear();
	}
}

void CMainFrame::OnFilterMaxEntries()
{
	CDialogMaxEntries dlg;
	dlg.MaxEntries = this->pList->Settings.Filter.MaxEntries;
	if(dlg.DoModal()==IDOK){
		this->pList->Settings.Filter.MaxEntries = dlg.MaxEntries;
		this->pList->RefreshLog();
		this->pDetails->Clear();
	}
}

void CMainFrame::OnFilterClear()
{
	this->pList->ClearFilters();
	this->pDetails->Clear();
}

bool CMainFrame::SetClipboardText(CString &Text)
{
	CClipboard c;
	return c.SetText(Text);
}

void CMainFrame::OnWhoisLookup() 
{
	WhoisLookup(pDetails->GetSelText());
}

void CMainFrame::WhoisLookup(CString &Query)
{
	CDialogWhois dlg;
	dlg.SetQuery(Query);
	dlg.SetServer(this->pList->Settings.General.WhoisServer);
	dlg.DoModal();
	this->pList->Settings.General.WhoisServer = dlg.GetServer();
}

void CMainFrame::OnDNSLookup() 
{
	DNSLookup(pDetails->GetSelText());
}

void CMainFrame::DNSLookup(CString &Query)
{
	CDialogDNS dlg;
	dlg.SetQuery(Query);
	dlg.DoModal();
}

void CMainFrame::SetSeparator()
{
	CDialogSeparator dlg;
	dlg.Separator = this->pList->Settings.General.Separator;
	if(dlg.DoModal()==IDOK){
		this->pList->Settings.General.Separator = dlg.Separator;
		this->pList->RefreshLog();
		this->pDetails->Clear();
	}
}

void CMainFrame::AddFilter(LPCTSTR Text)
{
	static CDialogFilter dlg; //needs to be static for dropdown list items
	dlg.FilterText = Text;
	if(dlg.DoModal()==IDOK){
		this->pList->AddFilter(dlg.FilterText,dlg.FilterType);
		this->pDetails->Clear();
	}
}

bool CMainFrame::OpenFile(LPCTSTR fn)
{
	Clear();

	CFileName path(fn);
	//settings in log folder
	if(CFileName::Exists(path.Path + "LogAnalysis.ini")){
		//AfxMessageBox(path.Path + "LogAnalysis.ini");
		this->pList->Settings.ReadFromFile(path.Path + "LogAnalysis.ini");
		SyncMenu();
	}

	this->m_wndToolBar.GetToolBarCtrl().EnableButton(ID_OPEN,FALSE);
	this->m_wndToolBar.GetToolBarCtrl().EnableButton(ID_FILTER_ADD,FALSE);
	this->m_wndToolBar.GetToolBarCtrl().EnableButton(ID_FILTER_EDIT,FALSE);

	AbortOpenFile = false;

	if (!pList->ReadLog(fn)){
		CString s;
		s.LoadString(IDS_APP_COULDNOTREADLOGFILE);
		AfxMessageBox(s);
		return false;
	}

	//sort again according to settings
	if(!AbortOpenFile && this->pList->Settings.General.SortedColumn>-1)
		this->pList->SortColumn(this->pList->Settings.General.SortedColumn,true);

	return !AbortOpenFile;
}

void CMainFrame::Clear()
{
	pList->CloseLog();
	pDetails->Clear();
	SetStatusBar(_T(""));
	SetStatusBarCount();
}

void CMainFrame::OnFilePrint() 
{
	pDetails->PrintDetails();
	//pList->PrintLog();
}

void CMainFrame::OnFilePrintPreview() 
{
	pDetails->PrintPreview();
	//pList->PrintPreview();
}

void CMainFrame::OnClose()
{
	if(DeferredFile.IsEmpty()){
		pList->AbortReading = true;
		AbortOpenFile = true;
		DoEvents();

		CFrameWnd::OnClose();
	}else{
		OnFileClose();
	}
}
void CMainFrame::OnTimer(UINT_PTR nIDEvent)
{
	if(nIDEvent == IDT_TIMER_DEFERRED){
		KillTimer(IDT_TIMER_DEFERRED);
		LoadDeferredFile();
		DoEvents();
	}else{
		CFrameWnd::OnTimer(nIDEvent);
	}
}
